import java.util.ArrayList;

public class Calculator
{

    /*************************************
    This class requires a formula (as a string).
    Then it will use the Formula_Parser to turn
    the string into a list of Elements. The Elements
    can then be used for calculations and a set
    of graph data can be given.
    **************************************/
    private ArrayList element_list;
    private ElementContainer root;
    private final int array_size = 1000;
    private double[][] combinations;
    private double[][] compacted;
    private double[][] normalised;
    
    /**
     * Constructor takes an array list of elements to perform calculations on.
     * @param element_list An array list of elements that will be used for calculations
     */
    public Calculator(ArrayList element_list) {
        this.element_list = element_list;
        root = containElements(element_list);
        combinations = root.calculate();
        compacted = compact(combinations);
        normalised = normalise(compacted);
    }
    
    /**
     * Takes an array list filled with elements
     * Returns an Element Container containing further
     * Element containers embeded inside each other
     * This will be used for itterating along the elements
     * and adding up their masses in every combination.
     * 
     * @param elemArray An array filled with Element objects.
     * @return An Element Container object filled with further Element Containers
     * embeded inside each other.
     */
    public ElementContainer containElements(ArrayList elemArray) {
        int i;
        
        ElementContainer currentElem = new ElementContainer();
        ElementContainer parent;
        
        for (i=elemArray.size()-1; i>0; i--) {
            currentElem.setElem( (Element)elemArray.get(i));
            parent = new ElementContainer();
            parent.setChild(currentElem);
            currentElem = parent;
        }
        currentElem.setElem((Element)elemArray.get(0));
        
        return currentElem;
    }
    
    /**
     * Takes an array of doubles and adds the percentages of
     * chemicals that have the same mass when rounded.
     * 
     * @param combinations A 2D array of doubles (mass mapped to percentage).
     * @return A 2D array of doubles (representing mass and respective abundance).
     */
    public double[][] compact(double[][] combinations)
    {
        int i, j;
        double unique_list[][] = new double[2][array_size];
        
        // Delete duplicate weights.
        for (i=0; i<combinations[0].length; i++)
        {
            for (j=0; j<combinations[0].length; j++)
            {
                if (i != j && Math.round(combinations[0][i]) == Math.round(combinations[0][j]))
                {
                    combinations[1][i] += combinations[1][j]; // Add percentages for same weights
                    combinations[0][j] = 0;
                    combinations[1][j] = 0;
                }
            }
        }
        
        // Now create a new array with no zeros and no duplicated masses.
        j=0; // We're going to re-use 'j' - it saves creating another variable.
        for (i=0; i<combinations[0].length; i++)
        {
            if (combinations[0][i] > 0)
            {
                // Save the non-zero, rounded values into the new array.
                unique_list[0][j] = (double)Math.round(combinations[0][i]);
                // Save the corresponding percentages too.
                unique_list[1][j] = combinations[1][i];
                j++;
            }
        }
        return unique_list;
    }
    
    /**
     * Takes a compacted array of masses and their percentages
     * and returns a new array of the masses with normalised
     * percentages.
     * 
     * @param compacted_array A 2D array of doubles that has been compacted.
     * @return A 2D array representing masses and normalised abundanced.
     */
    public double[][] normalise(double[][] compacted_array) {
        int i = 0;
        double max = 0;
        double[][] normalised_array = new double[2][array_size];
        
        // Find maximum percentage
        while (compacted_array[0][i] > 0) {
            if(compacted_array[0][i] > max) {
                max = compacted_array[1][i];
            }
            i++;
        }
        
        i=0;
        
        // Normalise percentages.
        while (compacted_array[0][i] > 0) {
            normalised_array[0][i] = compacted_array[0][i];
            normalised_array[1][i] = (compacted_array[1][i] / max) * 100;
            i++;
        }
        
        return normalised_array;
    }
    
    /**
     * Returns an array list containing all the points that a graph will need.
     * @param normalised_array An array that has had its percentages normalised.
     * @return An array list of graph points.
     */
    public ArrayList graphPoints(double[][] normalised_array) {
        int i=0, min, max, xPos;
        ArrayList points = new ArrayList();
        
        min = minMass(normalised_array);
        max = maxMass(normalised_array);

        while (normalised[0][i] > 0) {
            if ((double)normalised[0][i] != (double)min){
                xPos = (int)(((double)normalised[0][i] - min)/(max-min)*10);
            }
            else {
                xPos = 1;
            }
            points.add( new Point(xPos, (int)normalised[1][i]) );
            i++;
        }
        return points;
    }
    
    /**
     * Returns the biggest mass in the array.
     * @param array a 2D array of doubles.
     * @return The largest mass found in the array.
     */
    public int maxMass(double[][] array) {
        int i=0, max=0;
        
        while (normalised[0][i] > 0) {
            if (max < array[0][i]) {
                max = (int)array[0][i];
            }
            i++;
        }
        return max;
    }
    
    /**
     * Returns the smallest mass in the array.
     * @param array a 2D array of doubles.
     * @return The smallest mass found in the array.
     */
    public int minMass(double[][] array) {
        int i=0, min=0;
        
        min = (int) array[0][0];
        while (array[0][i] > 0) {
            if (min > array[0][i]) {
                min = (int)array[0][i];
            }
            i++;
        }
        return min;
    }
    
    /**
     * Returns a set of graph data.
     * 
     * @return Data to be displayed on a graph.
     */
    public GraphDataSet getGraphData() {
            int i=0, minMass=0, maxMass=0;
            String origin = "";
            String[] ylabels = new String[11], xlabels = new String[11];
            
            minMass = minMass(normalised);
            maxMass = maxMass(normalised);
            origin = Integer.toString(minMass - 1);
            
            // Generate % markings up the graph.
            ylabels[0] = "";
            for(i=1; i<11; i++) {
                ylabels[i] = Integer.toString(10 * i);
            }
            
            // Generate m/z markings along the graph.
            xlabels[0] = "";
            for(i=1; i<11; i++) {
                xlabels[i] = Double.toString( minMass/((maxMass-minMass)*10) /*(minMass) + ((maxMass-minMass)/10.0) * (i-1)    */);
            }
            
            return new GraphDataSet(xlabels, ylabels, origin, graphPoints(normalised));
        }
}