import javax.swing.*;
import java.awt.*;
import java.awt.event.*;

/**
 * GUI Class.
 * 
 * Creates the user interface and forwards events to the engine.
 * 
 * @author Team NAH
 * @version Spring 2006
 */
public class GUI extends JFrame implements ActionListener {

	// define class variables
	private Engine engine;

	private JTabbedPane tabbedPane;

	private JPanel graphPanel, databasePanel, settingsPanel, aboutPanel;

	private ImageIcon mainIcon;

	private JLabel aboutText, formulaTextLabel, errorLabel;

	private ChemGraph graph;

	private JTextField formulaText;

	private JButton formulaOkBtn;

	/**
	 * GUI constructor.
	 * 
	 * Sets up the look and feel to the native system, then gets a reference to
	 * the engine object.
	 */
	public GUI() {
		super();
		// try to set up the system look and feel
		try {
			UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
		} catch (Exception e) {
			ErrorSubject
					.quickError(0,
							"Look and feel setting failed, will use cross platform look");
			// do nothing if it doesn't work
			// -we'll use the cross platform look
		}
		// get singleton reference
		engine = Engine.getEngine();

		// insure the window is properly disposed of
		addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent e) {
				finalise();
			}
		});

	}
	
	/**
	 * Set the error label's values.
	 */
	public void setErrLabel(String label) {
		errorLabel.setText(label);
	}

	/*
	 * Actions to be performed when termination signal received.
	 * 
	 * Not to be confused with the finalize() method in Frame.
	 */
	private void finalise() {
		dispose();
		System.exit(0);
	}
	
	/**
	 * Get the current graph.
	 */
	 public ChemGraph getGraph() {
	 	return graph;
	 }
	 
	/**
	 * set the data set
	 */
	public void setData(GraphDataSet gds) {
		graph.setData(gds);
	}

	/**
	 * Assemble the GUI.
	 * 
	 * Does exactly what it says on the tin.
	 */
	public void assemble() {
		Container contentPane = getContentPane();
		contentPane.setLayout(new GridLayout(1, 1));

		// create and set up panels
		graphPanel = new JPanel();
		graphPanel.setLayout(new BorderLayout());
		databasePanel = new DatabaseInterface();
		aboutPanel = new JPanel();

		// continer to hold the formula textfield, label and button
		// (goes in the top half of the main panel
		Container formulaCont = new Container();
		formulaCont.setLayout(new FlowLayout());

		// container to hold the graph
		// (goes in the bottom half of the main panel
		Container graphCont = new Container();
		graphCont.setLayout(new FlowLayout());

		// container to hold the formula textfield and label
		Container formulaTextCont = new Container();
		formulaTextCont.setLayout(new GridLayout(1, 2));

		// container to hold the previous container with button under
		Container formulaTextOuterCont = new Container();
		formulaTextOuterCont.setLayout(new GridLayout(2, 1));

		// add things to the containers
		formulaText = new JTextField("C8H10N4O2");
		formulaText.setPreferredSize(new Dimension(100, 20));

		formulaTextLabel = new JLabel("Enter formula:");
		formulaOkBtn = new JButton("Calculate");
		formulaOkBtn.addActionListener(this);

		formulaTextCont.add(formulaTextLabel);
		formulaTextCont.add(formulaText);
		formulaTextOuterCont.add(formulaTextCont);
		formulaTextOuterCont.add(formulaOkBtn);

		formulaCont.add(formulaTextOuterCont);
		graphPanel.add(formulaCont, BorderLayout.NORTH);

		String xlabels[] = { "", "191", "192", "193", "194", "195", "196",
				"197", "198", "199", "200" };
		String ylabels[] = { "", "10 ", "20 ", "30 ", "40 ", "50 ", "60 ",
				"70 ", "80 ", "90 ", "100" };
		String origin = "0";

		GraphDataSet dataSet = new GraphDataSet(xlabels, ylabels, origin, null);

		Point aPoint = new Point(4, 100);
		Point bPoint = new Point(5, 99);
		Point cPoint = new Point(6, 50);
		Point dPoint = new Point(7, 25);
		dataSet.addPoint(aPoint);
		dataSet.addPoint(bPoint);
		dataSet.addPoint(cPoint);
		dataSet.addPoint(dPoint);

		graph = new ChemGraph(dataSet, 500, 500);

		graphCont.add(graph);

		graphPanel.add(graphCont, BorderLayout.SOUTH);

		// credits
		aboutText = new JLabel("<html><u>Spectre - A Mass Spectrometer "
				+ "Simulator in Java</u><br><br>Produced in Spring "
				+ "2006 at the University of Wales, Swansea.<br><br>"
				+ "Coding: <i>Karim Kanso, Roger Barnfather, Sean "
				+ "Handley</i><br>Documentation: <i>Lee Meader, John "
				+ "Arran, Chris Usher</i><br><br>"
				+ "Visit: http://spectre.zapto.org");
		aboutPanel.add(aboutText);

		// mainIcon = createImageIcon("MainIcon.png"); //maybe use icons later

		// add the previously crafted panels to the tabbed pane
		tabbedPane = new JTabbedPane();

		tabbedPane.setPreferredSize(new Dimension(625, 600));

		tabbedPane.addTab("Main", null, graphPanel, "Analyse a molecule");
		tabbedPane.addTab("Isotope Database", null, databasePanel,
				"Edit isotopes in the database");
		tabbedPane.addTab("About", null, aboutPanel, "View the credits");

		// create and add the error label, also give it a default text
		errorLabel = new UIErrorObserver();
		errorLabel.setText("Initialised OK");
		ErrorSubject.getErrorSubject().attachObserver(
				(UIErrorObserver) errorLabel);

		// add the pane to the main window
		contentPane.setLayout(new BorderLayout());
		contentPane.add(tabbedPane, BorderLayout.CENTER);
		contentPane.add(errorLabel, BorderLayout.SOUTH);

		pack();

		// centre it and display
		setLocationRelativeTo(null);
		setTitle("Spectre");
		setVisible(true);
	}

	/*
	 * Returns an ImageIcon, or null if the path was invalid
	 */
	private static ImageIcon createImageIcon(String path) {
		java.net.URL imgURL = GUI.class.getResource(path);
		if (imgURL != null) {
			return new ImageIcon(imgURL);
		} else {
			System.err.println("Couldn't find file: " + path);
			return null;
		}
	}

	/**
	 * Action event handler.
	 * 
	 * Responds to events caught by the action listener.
	 */
	public void actionPerformed(ActionEvent event) {
		if (event.getSource() == formulaOkBtn) {
			engine.formulaOkBtnPressed(formulaText.getText());
		}
	}
}
